<?php 
/**
 * Plugin Name: SmartLink Plugin
 * Plugin URI:  https://tajnews.org/ultimate-short-url-plugin/
 * Description: Automatically create and display short URLs using YOURLS. Works out of the box with default settings, but allows overriding via plugin settings.
 * Version:     1.0.0
 * Author:      Teshaev
 * Author URI:  https://smtlink.click
 * License:     GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: smartlink-plugin
 */

if ( ! defined( 'ABSPATH' ) ) {
    exit; // Prevent direct access
}

/* ----------------------------------------------------------------------
 * 1. Register settings for YOURLS domain & signature
 * ---------------------------------------------------------------------- */
function smartlink_register_settings() {
    register_setting( 'smartlink_options_group', 'smartlink_domain' );
    register_setting( 'smartlink_options_group', 'smartlink_signature' );
}
add_action( 'admin_init', 'smartlink_register_settings' );

function smartlink_settings_menu() {
    add_options_page(
        'SmartLink Settings',
        'SmartLink',
        'manage_options',
        'smartlink-settings',
        'smartlink_settings_page'
    );
}
add_action( 'admin_menu', 'smartlink_settings_menu' );

/**
 * 1.1. Settings page content
 */
function smartlink_settings_page() {
    if ( ! current_user_can( 'manage_options' ) ) {
        return;
    }

    $domain    = get_option( 'smartlink_domain', '' );
    $signature = get_option( 'smartlink_signature', '' );
    ?>
    <div class="wrap">
        <h1>SmartLink Settings</h1>
        
        <div style="margin-bottom: 20px; padding: 15px; background: #f8f8f8; border: 1px solid #ddd;">
            <h2>About this Plugin</h2>
            <p>
                <strong>SmartLink Plugin</strong> was initially created for our personal use, utilizing a default YOURLS server and signature. Due to the limitations of many link-shortening solutions, we decided to make this plugin publicly available <strong>for free</strong>.
            </p>
            <p>
                By default, the plugin works in <em>automatic mode</em> with our YOURLS server. If you prefer to use your own YOURLS domain and key, simply fill in the fields below and click "Save Changes".
            </p>

            <h3>Shortcode Usage</h3>
            <p>
                To display the short link manually in your post or page, use the shortcode:
                <br><code>[smart_link]</code><br>
                It will output the generated short URL along with "Copy" and "QR Code" buttons.
            </p>

            <h3>Bookmarklets & Prefix Shortening</h3>
            <p>
                YOURLS provides bookmarklets for easy link shortening and sharing:
                <ul style="list-style: disc; margin-left: 20px;">
                    <li><strong>Standard bookmarklet:</strong> opens a page to edit or delete your short URLs.</li>
                    <li><strong>Instant bookmarklet:</strong> shows a popup with the short URL without leaving the current page.</li>
                    <li><strong>Simple or Advanced:</strong> either generate a random short URL or choose your own custom keyword.</li>
                </ul>
                Simply drag these bookmarklets from your YOURLS admin panel to your browser’s bookmarks toolbar.
            </p>
            <p>
                <strong>Prefix-based Shortening:</strong> When viewing a page at <code>http://some-long-site.com/path</code>, you can add <code>smtlink.click/</code> before <em>http://</em> in the address bar (e.g. <code>smtlink.click/http://some-long-site.com/path</code>) and press Enter to instantly create a short URL.
            </p>
            
            <h3>Safe API Calls (Signature)</h3>
            <p>
                Instead of sending username/password, YOURLS allows the use of a secret signature token for API requests. This plugin uses that method to keep your credentials secure. For time-limited links, you may pass both <code>signature</code> and <code>timestamp</code> parameters.
            </p>

            <h3>How to Use Your Own Domain & Key</h3>
            <ol style="list-style: decimal; margin-left: 20px;">
                <li>Install and configure YOURLS on your own hosting (e.g. <code>https://short.example.com</code>).</li>
                <li>Retrieve your <strong>Signature Key</strong> from the YOURLS admin area.</li>
                <li>Enter your domain and signature key below. (Leave blank to use the default settings.)</li>
                <li>Click <strong>Save Changes</strong>. New posts will then receive a short link from your YOURLS server.</li>
            </ol>
        </div>

        <form method="post" action="options.php">
            <?php
            settings_fields( 'smartlink_options_group' );
            do_settings_sections( 'smartlink_options_group' );
            ?>
            <table class="form-table">
                <tr>
                    <th><label for="smartlink_domain">YOURLS Domain:</label></th>
                    <td>
                        <input type="text" id="smartlink_domain" name="smartlink_domain"
                               value="<?php echo esc_attr( $domain ); ?>"
                               size="50" placeholder="https://example.com" />
                        <p class="description">
                            Enter your YOURLS domain (include http/https, no trailing slash). Leave blank to use the default.
                        </p>
                    </td>
                </tr>
                <tr>
                    <th><label for="smartlink_signature">Signature Key:</label></th>
                    <td>
                        <input type="text" id="smartlink_signature" name="smartlink_signature"
                               value="<?php echo esc_attr( $signature ); ?>"
                               size="50" placeholder="yourlsAPIkey123" />
                        <p class="description">
                            The secret signature for YOURLS API requests. Leave blank to use the default.
                        </p>
                    </td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

/* ----------------------------------------------------------------------
 * 2. Create short link via YOURLS API with fallback
 * ---------------------------------------------------------------------- */
function smartlink_create_shortlink( $long_url ) {
    // Получаем настройки, либо используем значения по умолчанию
    $domain    = get_option( 'smartlink_domain', '' );
    $signature = get_option( 'smartlink_signature', '' );

    if ( empty( $domain ) ) {
        $domain = 'https://smtlink.click';
    }
    if ( empty( $signature ) ) {
        $signature = '6a484a3a49';
    }

    $api_url = rtrim( $domain, '/' ) . '/yourls-api.php';
    $response = wp_remote_post( $api_url, array(
        'body'    => array(
            'signature' => $signature,
            'action'    => 'shorturl',
            'format'    => 'json',
            'url'       => $long_url,
        ),
        'timeout' => 10,
    ) );

    if ( is_wp_error( $response ) ) {
        return false;
    }

    $body = wp_remote_retrieve_body( $response );
    $data = json_decode( $body, true );

    if ( isset( $data['shorturl'] ) ) {
        return $data['shorturl'];
    }
    return false;
}

/* ----------------------------------------------------------------------
 * 3. Автогенерация короткой ссылки при публикации поста
 * ---------------------------------------------------------------------- */
function smartlink_autoshortlink_on_publish( $post_id ) {
    if ( wp_is_post_revision( $post_id ) ) {
        return;
    }

    // Используем новый мета-ключ для короткой ссылки
    $existing_short = get_post_meta( $post_id, '_smartlink_url', true );
    if ( ! empty( $existing_short ) ) {
        return;
    }

    $long_url  = get_permalink( $post_id );
    $short_url = smartlink_create_shortlink( $long_url );
    if ( $short_url ) {
        update_post_meta( $post_id, '_smartlink_url', $short_url );
    }
}
add_action( 'publish_post', 'smartlink_autoshortlink_on_publish' );

/* ----------------------------------------------------------------------
 * 4. Шорткод [smart_link]: выводит короткую ссылку + кнопки "Copy" и "QR Code"
 * ---------------------------------------------------------------------- */
function smartlink_shortcode() {
    $post_id = get_the_ID();
    if ( ! $post_id ) {
        return '';
    }

    $short_url = get_post_meta( $post_id, '_smartlink_url', true );
    // Если короткая ссылка отсутствует (для старых публикаций), генерируем её
    if ( empty( $short_url ) ) {
        $long_url  = get_permalink( $post_id );
        $short_url = smartlink_create_shortlink( $long_url );
        if ( $short_url ) {
            update_post_meta( $post_id, '_smartlink_url', $short_url );
        }
    }

    if ( ! empty( $short_url ) ) {
        ob_start();
        ?>
        <div class="smartlink-box">
            <strong>Short URL:</strong>
            <span style="margin-left: 10px;"><?php echo esc_html( $short_url ); ?></span>
            <a href="#" style="margin-left: 10px;"
               onclick="smartlinkCopyShortLink('<?php echo esc_js( $short_url ); ?>'); return false;">
               Copy
            </a>
            <a href="#" style="margin-left: 10px;"
               onclick="smartlinkToggleQRCode(this, '<?php echo esc_js( $short_url ); ?>'); return false;">
               QR Code
            </a>
            <div class="smartlink-qr-code" style="display:none; margin-top:10px;">
                <img src="" alt="QR Code" style="max-width:150px;">
            </div>
        </div>
        <?php
        return ob_get_clean();
    }
    return '';
}
add_shortcode( 'smart_link', 'smartlink_shortcode' );

/* ----------------------------------------------------------------------
 * 5. Метабокс в редакторе: отображает короткую ссылку (нажмите для копирования/открытия)
 * ---------------------------------------------------------------------- */
function smartlink_add_meta_box() {
    add_meta_box(
        'smartlink_box',
        'SmartLink',
        'smartlink_render_meta_box',
        'post',
        'side',
        'low'
    );
}
add_action( 'add_meta_boxes', 'smartlink_add_meta_box' );

function smartlink_render_meta_box( $post ) {
    $short_url = get_post_meta( $post->ID, '_smartlink_url', true );
    if ( ! empty( $short_url ) ) {
        ?>
        <p><strong>Short URL:</strong></p>
        <p>
            <a href="<?php echo esc_url( $short_url ); ?>"
               onclick="smartlinkCopyShortLink('<?php echo esc_js( $short_url ); ?>'); return false;"
               target="_blank">
               <?php echo esc_html( $short_url ); ?>
            </a>
        </p>
        <?php
    } else {
        echo '<p>No short link yet (it will appear after publishing).</p>';
    }
}

/* ----------------------------------------------------------------------
 * 6. Автоматическое добавление короткой ссылки в содержимое поста
 * ---------------------------------------------------------------------- */
function smartlink_append_shortlink_at_bottom( $content ) {
    if ( is_singular( 'post' ) && in_the_loop() && is_main_query() ) {
        $post_id   = get_the_ID();
        $short_url = get_post_meta( $post_id, '_smartlink_url', true );

        if ( ! empty( $short_url ) ) {
            $html  = '<div class="smartlink-box" style="margin-top: 20px; padding: 10px; border-top: 1px solid #ddd;">';
            $html .= '<strong>Short URL:</strong> ';
            $html .= '<span style="margin-left: 10px;">' . esc_html( $short_url ) . '</span>';
            $html .= '<a href="#" style="margin-left: 10px;" onclick="smartlinkCopyShortLink(\'' . esc_js( $short_url ) . '\'); return false;">Copy</a>';
            $html .= '<a href="#" style="margin-left: 10px;" onclick="smartlinkToggleQRCode(this, \'' . esc_js( $short_url ) . '\'); return false;">QR Code</a>';
            $html .= '<div class="smartlink-qr-code" style="display:none; margin-top:10px;"><img src="" alt="QR Code" style="max-width:150px;"></div>';
            $html .= '</div>';

            $content .= $html;
        }
    }
    return $content;
}
add_filter( 'the_content', 'smartlink_append_shortlink_at_bottom' );

/* ----------------------------------------------------------------------
 * 7. JavaScript для "Copy to clipboard" и переключения QR Code
 * ---------------------------------------------------------------------- */
function smartlink_copy_script() {
    if ( is_singular( 'post' ) ) {
        ?>
        <script>
        function smartlinkCopyShortLink(url) {
            if (navigator.clipboard && window.isSecureContext) {
                navigator.clipboard.writeText(url).then(function() {
                    alert('Link copied: ' + url);
                }, function(err) {
                    alert('Copy error: ' + err);
                });
            } else {
                // Фоллбэк для старых браузеров
                var textArea = document.createElement('textarea');
                textArea.value = url;
                textArea.style.position = 'fixed';
                textArea.style.left = '-999999px';
                document.body.appendChild(textArea);
                textArea.focus();
                textArea.select();
                try {
                    if (document.execCommand('copy')) {
                        alert('Link copied: ' + url);
                    } else {
                        alert('Could not copy the link automatically.');
                    }
                } catch (e) {
                    alert('Copy exception: ' + e);
                }
                textArea.remove();
            }
        }

        function smartlinkToggleQRCode(el, url) {
            var container = el.parentElement.querySelector('.smartlink-qr-code');
            if (!container) return;
            if (container.style.display === 'none' || container.style.display === '') {
                var img = container.querySelector('img');
                if (!img.getAttribute('src')) {
                    img.setAttribute('src', 'https://api.qrserver.com/v1/create-qr-code/?size=150x150&data=' + encodeURIComponent(url));
                }
                container.style.display = 'block';
            } else {
                container.style.display = 'none';
            }
        }
        </script>
        <?php
    }
}
add_action( 'wp_footer', 'smartlink_copy_script' );
